@@ -1,3 +1,22 @@
+/* gnome-python - python bindings for gnome-libs
+ * Copyright (C) 1998-1999 James Henstridge <james@daa.com.au>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <Python.h>
 #include <sysmodule.h>
 
@@ -164,7 +183,7 @@ static GnomeUIInfo *list_to_ui_info(PyObject *list, gboolean inc_uibd) {
   
   for (i = 0; i < len; i++) {
     item = PyList_GetItem(list, i);
-    if (!PyTuple_Check(item) || !PyArg_ParseTuple(item, "izzOOizcO",
+    if (!PyTuple_Check(item) || !PyArg_ParseTuple(item, "izzOOiziO",
 						  &(ret[i].type),
 						  &(ret[i].label),
 						  &(ret[i].hint),
@@ -184,7 +203,9 @@ static GnomeUIInfo *list_to_ui_info(PyObject *list, gboolean inc_uibd) {
     else
       Py_INCREF((PyObject *)ret[i].user_data);
     type = ret[i].type; pix_type = ret[i].pixmap_type;
-    if (type < GNOME_APP_UI_ENDOFINFO || type > GNOME_APP_UI_HELP ||
+    if (type < GNOME_APP_UI_ENDOFINFO ||
+	type == GNOME_APP_UI_BUILDER_DATA ||
+	type > GNOME_APP_UI_SUBTREE_STOCK ||
 	pix_type < GNOME_APP_PIXMAP_NONE ||
 	pix_type > GNOME_APP_PIXMAP_FILENAME ||
 	pix_type == GNOME_APP_PIXMAP_DATA) {
@@ -223,6 +244,7 @@ static GnomeUIInfo *list_to_ui_info(PyObject *list, gboolean inc_uibd) {
       ret[i].moreinfo = sub;
       break;
     case GNOME_APP_UI_SUBTREE:
+    case GNOME_APP_UI_SUBTREE_STOCK:
       sub = list_to_ui_info(moreinfo, FALSE);
       if (sub == NULL) {
 	g_free(inf);
@@ -241,6 +263,21 @@ static GnomeUIInfo *list_to_ui_info(PyObject *list, gboolean inc_uibd) {
       else
 	ret[i].moreinfo = PyString_AsString(moreinfo);
       break;
+    case GNOME_APP_UI_ITEM_CONFIGURABLE:
+      if (moreinfo == Py_None) {
+        ret[i].moreinfo = NULL;
+        break;
+      }
+      if (!PyCallable_Check(moreinfo)) {
+	sprintf(buf, "additional info not callable (label \"%s\")",
+		ret[i].label);
+	PyErr_SetString(PyExc_TypeError, buf);
+	g_free(inf);
+	return NULL;
+      }
+      ret[i].moreinfo = moreinfo;
+      Py_INCREF(moreinfo);
+      break;
     }
   }
   return inf;
@@ -255,7 +292,8 @@ static void free_ui_info(GnomeUIInfo *inf) {
       if (inf[i].moreinfo !=  NULL)
 	free_ui_info(inf[i].moreinfo);
     if ((inf[i].type == GNOME_APP_UI_ITEM ||
-	 inf[i].type == GNOME_APP_UI_TOGGLEITEM) &&
+	 inf[i].type == GNOME_APP_UI_TOGGLEITEM ||
+	 inf[i].type == GNOME_APP_UI_ITEM_CONFIGURABLE) &&
 	inf[i].moreinfo != NULL)
       Py_DECREF((PyObject *)inf[i].moreinfo);
     if (inf[i].user_data != NULL)
@@ -439,6 +477,19 @@ static PyObject *_wrap_gnome_app_install_menu_hints(PyObject *self, PyObject *ar
   return Py_None;
 }
 
+static PyObject *_wrap_gnome_canvas_new_aa(PyObject *self, PyObject *args) {
+  GtkWidget *ret;
+
+  if (!PyArg_ParseTuple(args, ":gnome_canvas_new_aa"))
+    return NULL;
+  gtk_widget_push_colormap(gdk_rgb_get_cmap());
+  gtk_widget_push_visual(gdk_rgb_get_visual());
+  ret = gnome_canvas_new_aa();
+  gtk_widget_pop_visual();
+  gtk_widget_pop_colormap();
+  return PyGtk_New((GtkObject *)ret);
+}
+
 static PyObject *_wrap_gnome_canvas_get_scroll_region(PyObject *self,
 						 PyObject *args) {
   PyObject *obj;
@@ -1401,6 +1452,7 @@ static PyMethodDef gnomeuiMethods[] = {
     { "gnome_app_create_toolbar", _wrap_gnome_app_create_toolbar, 1 },
     { "gnome_app_insert_menus", _wrap_gnome_app_insert_menus, 1 },
     { "gnome_app_install_menu_hints", _wrap_gnome_app_install_menu_hints, 1 },
+    { "gnome_canvas_new_aa", _wrap_gnome_canvas_new_aa, 1 },
     { "gnome_canvas_get_scroll_region", _wrap_gnome_canvas_get_scroll_region, 1 },
     { "gnome_canvas_get_scroll_offsets", _wrap_gnome_canvas_get_scroll_offsets, 1 },
     { "gnome_canvas_w2c", _wrap_gnome_canvas_w2c, 1 },
